
#include "BluetoothSerial.h"

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

BluetoothSerial SerialBT;

#include <Wire.h>
#include <Adafruit_MPU6050.h>

// Подключаем внутренние программные файлы.
#include "defin.h"  //Переменные и константы
#include "MadgwickAHRS.h" //Фильтр Маджвика
#include "core_motorstep.h"  //Описание моторов
#include "core_sonar.h" // Настрока сонара 
#include "core_servo_hand.h" //Обмен командами
#include "core_irq_robot.h"  //Работа с прерываниями
#include "CalcSpeed.h" //Пересчет шагов
#include "move_case.h" //Обмен командами со смартфоном

//=====================================
//+++++++++++++++++++++++++++++++++++++
//=====================================
bool error_Flag = false;


//IBusBM IBus; // IBus object
//HardwareSerial FSkySerial(2); //rx in 16gpio

Adafruit_MPU6050 mpu;
sensors_event_t a, g, temper;

#include "storage.h"

uint32_t  t2, t0;
//Задаем GPIO для Wire
const int _SDA = 21;
const int _SCL = 22;
void setup()
{
  servo_hand_setup();
  sonar_setup();
  setup_motor_system();
  //Выключаем мотор перемещения

  //Wire.setPins(_SDA,_SCL); //можно использовать в 5й версии
  Wire.begin(_SDA, _SCL);
  Serial.begin(115200);

  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");

  mpu.setAccelerometerRange(MPU6050_RANGE_8_G);
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);//MPU6050_RANGE_1000_DEG
  //mpu.setFilterBandwidth(MPU6050_BAND_21_HZ); //MPU6050_BAND_94_HZ // MPU6050_BAND_184_HZ
  //MPU6050_BAND_260_HZ, ///< Docs imply this disables the filter
  //MPU6050_BAND_184_HZ, ///< 184 Hz
  //MPU6050_BAND_94_HZ,  ///< 94 Hz
  mpu.setFilterBandwidth(MPU6050_BAND_260_HZ);
  delay(1000);
  Calc_CompensatorZ(5000);
  arming = false;
  timer_setup();
  Serial.print("start2");
  Serial.print("start3=");
  Serial.println(portTICK_PERIOD_MS);//configMAX_PRIORITIES);

  mpu.getEvent(&a, &g, &temper);
  commandRIGHT_WEEL_SPEED = 0;
  commandLEFT_WEEL_SPEED = 0;
  OldTime =  micros();
  Pereschet = millis() + 3000;
  SerialBT.begin("ESP32BALROBOT"); //Bluetooth device name
}

uint32_t Dtime = 5000;
uint32_t start_period = 0;
double dta = 0;

void loop()
{
  //Получаем от приемника два значения
  // 1. Командный угол поворота newROT_Angel_RAD - в радианах
  // 2. Командную скорость newWheelSpeed - в шагах на секунду
  BT_input();
  PrintDebug();
  
  micros_1 = micros();
  if ((micros_1 - OldTime) < 5000) return;

  Dtime = micros_1 - OldTime;
  Dtime = constrain(Dtime, 4000, 10000);
  Dt = double(Dtime) * 0.000001;
  OldTime = micros_1;

  mpu.getEvent(&a, &g, &temper);
  //Фильтр Маджвика
  MadgwickAHRSupdateIMU(Dt, g.gyro.x - CompensatorX , g.gyro.y - CompensatorY, g.gyro.z - CompensatorZ, a.acceleration.x, a.acceleration.y, a.acceleration.z);
  quat[0] = q0; quat[1] = q1; quat[2] = q2; quat[3] = q3;
  if ((millis() < Pereschet) && (!avtostart)) return;
  else avtostart = true;

  // преобразуем кватернион в углы Эйлера
  quat2Euler(&quat[0], &imu[0]);
  if (fabs(imu[1]) > 0.75) arming = false;
  else if ((fabs(imu[1]) < 0.55) && (!arming)){ arming = true; }

  opros_sys_EN_motors(); //Отключаем моторы в случае необходимости

  if (STEPER_EN_level == MOTORS_ON)
  {
    Error_ang = imu[1] - base_ang; //Ошибка угла наклона в радианах
    dError_ang = (Error_ang - OldError_ang) / Dt; //Скорость изменения ошибки угла наклона
    Error_speed = Sbo - CommandSpeed; //Ошибка скорости в шагах
    dError_speed = (Error_speed - OldError_speed) / Dt; //Скорость изменения ошибки скорости
    iError_speed += Error_speed * Dt * Ki_s; //Накопленная - интегральная ошибка скорости

    iError_speed = constrain(iError_speed, -2 * MAX_WEEL_SPEED, 2 * MAX_WEEL_SPEED);

    New_Speed = Error_ang * Kp_a + dError_ang * Kd_a + Error_speed * Kp_s + dError_speed * Kd_s + iError_speed;
    OldError_ang = Error_ang;
    OldError_speed = Error_speed;
    commandRIGHT_WEEL_SPEED = constrain(New_Speed - Turn, commandRIGHT_WEEL_SPEED - Dt * ACCELERATION_WEEL, commandRIGHT_WEEL_SPEED + Dt * ACCELERATION_WEEL);
    commandLEFT_WEEL_SPEED = constrain(New_Speed + Turn, commandLEFT_WEEL_SPEED - Dt * ACCELERATION_WEEL, commandLEFT_WEEL_SPEED + Dt * ACCELERATION_WEEL);
    OldSpeed = (commandLEFT_WEEL_SPEED + commandRIGHT_WEEL_SPEED) * 0.5 ;
  }
  else
  {
    New_Speed = OldSpeed = 0;
    commandRIGHT_WEEL_SPEED = 0;
    commandLEFT_WEEL_SPEED = 0;
  }
  CalcSpeedAndPosition(Dt);
}


void PrintDebug()
{
  static uint32_t printTime = 0;
  if (printTime < millis())
  {
    Serial.println("=");
    Serial.print("X=");
    Serial.println(imu[1]); //Serial.print("  acc="); Serial.print(a.acceleration.x);  Serial.print("  gyro=");  Serial.println(g.gyro.x);
    Serial.print("Y=");
    Serial.println(imu[0]); //Serial.print("  acc="); Serial.print(a.acceleration.y);  Serial.print("  gyro=");  Serial.println(g.gyro.y);
    Serial.print("Z=");
    Serial.println(imu[2]); //Serial.print("  acc="); Serial.print(a.acceleration.z);  Serial.print("  gyro=");  Serial.println(g.gyro.z);
    // Serial.print("flag_sonar=");Serial.println(flag_sonar);
    Serial.print("arming=");
    Serial.println(arming);
    Serial.print("dError_ang="); Serial.println(dError_ang);
    printTime = millis() + 200;
  }
}
